;;; scim-bridge-ja.el

;; Copyright (C) 2008 S. Irie

;; Author: S. Irie
;; Maintainer: S. Irie
;; Keywords: Input Method, i18n

(defconst scim-bridge-ja-version "0.6.9")

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 2, or
;; (at your option) any later version.

;; It is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
;; License for more details.

;; You should have received a copy of the GNU General Public
;; License along with this program; if not, write to the Free
;; Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
;; MA 02111-1307 USA

;;; Commentary:

;; このプログラムは、Emacs のための scim-bridge クライアントである
;; scim-bdidge.el に追加することにより、変数や関数の説明文字列を日本語の
;; 文章に差し替え、またいくつかの日本語向けの便利なコマンドを提供します。
;; scim-brifge.el は公式の scim-bridge の一部ではありません。

;; SCIM (Smart Common Input Method) は 30以上の言語 (CJK と多くの
;; ヨーロッパ言語) をサポートする POSIX スタイルのオペレーティングシステム
;; (Linux や BSD) のためのインプットメソッド (IM) プラットフォームです。

;;
;; Installation:
;;
;; 初めに、このファイル (scim-bridge-ja.el) 及び scim-bridge.el を
;; パスの通ったディレクトリに保存し、scim-bridge.el をバイトコンパイル
;; して下さい。
;;
;; 以下の行を .emacs ファイルに追加します。
;;
;;   (require 'scim-bridge-ja)
;;
;; Emacs を起動するときは、コマンドラインから以下のように打ち込んで下さい。
;;
;;   XMODIFIERS=@im=none emacs
;;
;; 起動したら、scim-mode を ON にします。
;;
;;   M-x scim-mode
;;
;;
;; .emacs ファイルの設定例です。
;;
;;   (require 'scim-bridge-ja)
;;   ;; C-SPC は Set Mark に使う
;;   (scim-define-common-key ?\C-\  nil)
;;   ;; C-/ は Undo に使う
;;   (scim-define-common-key ?\C-/ nil)
;;   ;; 変換候補ウインドウの横方向の位置を微調整する
;;   (setq scim-adjust-window-x-position 'gnome)
;;   ;; 起動時に scim-mode を自動的に ON にする
;;   (scim-mode t)
;;
;; 以下は日本語向けの便利な設定です。.emacs ファイルに追加します。
;;
;;   ;; SCIM-Anthy 使用時に、M-w しなくても再変換できるようにする
;;   (global-set-key [henkan] 'scim-anthy-reconvert-region)
;;   ;; アスキーモードのままローマ字入力してしまった時に、プリエディットに入れ直す
;;   (global-set-key [C-henkan] 'scim-transfer-romaji-into-preedit)
;;
;;
;; 注意：このプログラムは GNU Emacs 22 以降のバージョンで動作します。
;; また、端末エミュレータ上では動作しません。
;;

;; History:
;; 2008-09-08  S. Irie
;;         * Modify translation
;;         * Version 0.6.9
;;
;; 2008-09-05  S. Irie
;;         * Version 0.6.8
;;
;; 2008-07-26  S. Irie
;;         * Some unimportant changes
;;         * Version 0.6.7
;;
;; 2008-06-12  S. Irie
;;         * Add/modify translations
;;         * Version 0.6.6
;;
;; 2008-06-05  S. Irie
;;         * Change specification for `scim-transfer-romaji-into-preedit'
;;         * Version 0.6.5
;;
;; 2008-06-01  S. Irie
;;         * Add some convenient commands for Japanese IM
;;         * Bug fix
;;         * Version 0.6.4
;;
;; 2008-05-24  S. Irie
;;         * Add translations
;;         * Bug fix
;;         * Version 0.6.3
;;
;; 2008-05-22  S. Irie
;;         * Modify description of kana-RO key
;;         * Version 0.6.2
;;
;; 2008-05-20  S. Irie
;;         * Add/modify translations
;;         * Version 0.6.1
;;
;; 2008-05-18  S. Irie
;;         * Experimental first release
;;         * Version 0.6.0
;;
;; 2008-05-15  S. Irie
;;         * Version 0.5.0

;; ToDo:

;;; Code:

(require 'scim-bridge)
(require 'pc-select)

;; Variables
(defvar scim-anthy-reconversion-event 'S-henkan
  "この変数には、SCIM-Anthy のキーバインド設定で「再変換」に割り当てられて
いるキーイベントを設定してください。関数 `scim-anthy-reconvert-region'
で使用されます。")

(defvar scim-toggle-input-method-event 'zenkaku-hankaku
  "この変数には、SCIM のキーバインド設定で「SCIM 開始」に割り当てられて
いるキーイベントを設定してください。関数 `scim-transfer-romaji-into-preedit'
で使用されます。")

;; Functions
(defun scim-anthy-reconvert-region ()
  "SCIM-Anthy の使用時に、バッファ内の文字列を再変換します。
マークが活性ならばマークとポイントの位置を変換範囲の両端とし、そうでなければ
ポイントの位置にある文節が変換対象となります。再変換のためのキーイベントは、
変数 `scim-anthy-reconversion-event' で指定されます。"
  (interactive "*")
  (if mark-active
      (call-interactively 'copy-region-as-kill-nomark)
    (unless (string-match
	     (char-to-string (following-char))
	     "。、．，？！）」)]}\n")
      (while (and (forward-word)
		  (string-match "[ぁ-ん][ー・]"
				(concat (list (preceding-char)
					      (following-char)))))))
    (let ((end (point)))
      (while (and (backward-word)
		  (string-match "[ぁ-ん][ー・]"
				(concat (list (preceding-char)
					      (following-char))))))
      (copy-region-as-kill (point) end)))
  (setq unread-command-events
	(cons scim-anthy-reconversion-event unread-command-events)))

(defun scim-transfer-romaji-into-preedit (end-point)
  "バッファ内の指定位置にあるローマ字の文字列を削除すると同時に、その文字列
をキーイベントキューの先頭に追加し、SCIM をトグルします。この関数を用いると、
バッファ内の文字列をキー入力として SCIM に渡すことができます。
END-POINT は文字列の片端のバッファ内位置を指定します。インタラクティブに呼び
出された場合はポイント位置の値が代入されます。マークが活性であれば、対象となる
文字列はマークと END-POINT の位置を両端とし、そうでなければ END-POINT から
行の先頭に向かってローマ字列の開始位置を検索します。SCIM を切り換えるための
キーイベントは、変数 `scim-toggle-input-method-event' で指定されます。"
  (interactive "d*")
  (goto-char end-point)
  (let* ((beg (if mark-active
		  (mark)
		(let ((case-fold-search nil))
		  (while (string-match
			  "[a-z-,.!?]"
			  (char-to-string (preceding-char)))
		    (backward-char))
		  (point))))
	 (str (buffer-substring-no-properties beg end-point)))
    (delete-region beg end-point)
    (setq unread-command-events
	  (cons scim-toggle-input-method-event
		(append str unread-command-events)))))

(defun scim-replace-function-doc (symbol string)
  "SYMBOL の関数定義に関する説明文字列を STRING で置換します。"
  (let ((func (symbol-function symbol)))
    (if (byte-code-function-p func)
	(let ((new-func (append func nil)))
	  (setcar (nthcdr 4 new-func) string)
	  (fset symbol (apply 'make-byte-code new-func)))
      (setcar (nthcdr 2 func) string))))

;; Translate documentation strings
(put 'scim
     'group-documentation
     "洗練された多言語入力プラットフォーム")

(put 'scim-mode
     'variable-documentation
  "scim-mode の ON/OFF を切り替えます。
この変数に値を直接代入しても正しい動作はしませんので、
 \\[customize] あるいは関数 `scim-mode' を用いてください。")

(put 'scim-basic
     'group-documentation
     "基本的な入力方法の設定（モードの切り替え、キーボードなど）")

(put 'scim-mode-local
     'variable-documentation
     "この値が nil 以外ならば、入力のモードを各バッファごとに切り替えること
ができるように、IMコンテクストが各々のバッファについて登録されます。そうで
なければ、入力のモードはグローバルに切り替えられます。")

(put 'scim-common-function-key-list
     'variable-documentation
     "直接入力時及びプリエディット時において、SCIM がどのキーイベントを
受けとるかを表すリストです。このリストへの要素の追加・削除は、関数
`scim-define-common-key' を用いて行うこともできます。
注意：このオプションでは ESC や C-x などのプレフィックスキーを設定しないで
ください。もし設定すると、Emacs が操作不能になるかもしれません。")

(put 'scim-preedit-function-key-list
     'variable-documentation
     "プリエディット領域が存在する時に、SCIM がどのキーイベントを
受けとるかを表すリストです。このリストへの要素の追加・削除は、関数
`scim-define-preedit-key' を用いて行うこともできます。")

(put 'scim-use-kana-ro-key
     'variable-documentation
     "もし jp-106 キーボードを用いて日本語のかな入力方式を行うなら、この
オプションをオン(nil 以外)にして、シフトキーを押さなくても仮名文字の「ろ」
を入力できるようにしてください。
　このオプションは、Xウインドウシステムの設定をシェルコマンドの `xmodmap' を
用いて一時的に書き換えることによって有効化されます。")

(put 'scim-key-release-delay
     'variable-documentation
     "もし jp-106 キーボードを用いて SCIM-Anthy で日本語の親指シフト入力方式
を行うなら、SCIM に対して「キーを離した」という情報を送るまでの遅延時間をこの
オプションで設定してください（単位は秒）。値は SCIM-Anthy の同時打鍵時間の
設定値以下にセットしてください。")
(put 'scim-key-release-delay
     'custom-type
     '(choice (const :tag "なし" nil)
	      (number :tag "遅延時間（秒）"
		      :value 0.1)))

(put 'scim-undo-by-committed-string
     'variable-documentation
     "この値が nil ならば、アンドゥを行うときに短い確定済み文字列が20文字を
越えない範囲で連結され、逆に長い確定済み文字列は20文字ごとに分割されます。
値が nil 以外であれば、アンドゥは確定した文字列ごとに行われます。")

(put 'scim-clear-preedit-when-unexpected-event
     'variable-documentation
     "この値が nil 以外ならば、プリエディット中に予期せぬイベントが発生した
場合に、プリエディット領域が消去されます。予期せぬイベントとは、例えばマウス
を使った文字列の貼り付けなどです。")

;; Appearance
(put 'scim-appearance
     'group-documentation
  "外観の設定（フェイス、変換候補ウインドウなど）")

(put 'scim-preedit-default-face
  'face-documentation
  "このフェイスはプリエディット領域の全体を表します。")

(put 'scim-preedit-underline-face
  'face-documentation
  "「下線」で使われるフェイスです。")

(put 'scim-preedit-highlight-face
  'face-documentation
  "「強調」で使われるフェイスです。")

(put 'scim-preedit-reverse-face
  'face-documentation
  "「反転」で使われるフェイスです。")

(put 'scim-adjust-window-x-position
     'variable-documentation
     "このオプションは、変換候補ウインドウの位置を、ウインドウ内の変換候補が
プリエディット領域の変換候補と垂直方向に並ぶように調整するかどうかということを
指定します。この値が `gnome' ならば、GNOMEデスクトップ環境でのフォントサイズ
の設定値を用いて調整します。一方、値が整数で与えられると、その数はウインドウを
通常の位置からずらす量をピクセル数で表します。
　このオプションは、候補ウインドウが常時表示されるようなタイプの入力方法、
例えば SCIM-pinyin（中国語）等には適していません。なぜなら、入力位置が
画面の下の方にある時には候補ウインドウがカーソルを覆い隠してしまう可能性が
あるからです。")
(put 'scim-adjust-window-x-position
     'custom-type
     '(choice (const :tag "GNOME のフォント設定を利用" gnome)
	      (integer :tag "ピクセル数を指定" :value 24)
	      (const :tag "off" nil)))

(put 'scim-adjust-window-y-position
     'variable-documentation
     "この値が nil 以外ならば、変換候補ウインドウの垂直方向の表示位置が、シェル
コマンドの `xwininfo' を利用して調節されます。そうでなければ調整は行われず、
ウインドウが正確な位置よりも少し下にずれるかもしれません。")

(put 'scim-prediction-window-position
     'variable-documentation
     "(日本語入力時のみ) この変数の値は (POS . ADJ) というコンスセルの形で与
えられます。もし POS の値が が nil 以外ならば、予測候補ウインドウがプリエディ
ット領域の先頭部分の下に表示されます。ADJ の値が nil 以外ならば、水平方向の
位置が変換候補ウインドウと同じように微調整されます。")
(put 'scim-prediction-window-position
     'custom-type
     '(cons
       (choice :tag "位置（POS)"
	       (const :tag "プリエディット領域の末尾" nil)
	       (const :tag "プリエディット領域の先頭" t))
       (choice :tag "微調整（ADJ）"
	       (const :tag "変換候補ウインドウと同様にする" t)
	       (const :tag "off" nil))))


(put 'scim-mode-line-string
     'variable-documentation
     "この変数はマイナーモードを許可するかどうかを制御します。先頭の文字が
空白である値は短い文字列で、モードラインに表示されます。この文字列は、複数の
モードの文字列が同時に表示できるように、短くしなければなりません。")

;; Advanced settings
(put 'scim-expert
     'group-documentation
     "高度な設定")

(put 'scim-focus-update-interval
     'variable-documentation
     "ウインドウのフォーカスを監視する周期を秒単位で指定します。")

(put 'scim-focus-update-interval-long
     'variable-documentation
     "ウインドウのフォーカスの取得が `xprop' コマンドではできずに `xwininfo'
コマンドで代用される場合、`scim-focus-update-interval' ではなくこの値が監視
の周期として用いられます。")

(put 'scim-kana-ro-x-keysym
     'variable-documentation
     "日本語の「ろ」キーが使用される場合、Xウインドウシステムにおいてこのキー
に割り当てる KeySym の名前を指定します。このプログラムはバックスラッシュキー
を円記号(￥)キーと区別するために、バックスラッシュキーに代替の KeySym を設定
します。")

(put 'scim-kana-ro-key-symbol
     'variable-documentation
     "日本語の「ろ」キーが使用される場合、`scim-kana-ro-x-keysym' によって
このキーに与えられた KeySym に対応するイベントのシンボルを指定します。
このプログラムはバックスラッシュキーを円記号(￥)キーと区別するために、
バックスラッシュキーに代替の KeySym を設定します。")
(put 'scim-kana-ro-key-symbol
     'custom-type
     '(choice (symbol :tag "シンボル")
	      (const :tag "なし" nil)))

(put 'scim-bridge-timeout
     'variable-documentation
     "特別な理由がない限り、この設定は変更しないでください。")

(put 'scim-bridge-wait-reply
     'variable-documentation
     "特別な理由がない限り、この設定は変更しないでください。")

(put 'scim-bridge-socket-use-script
     'variable-documentation
     "この値が nil 以外ならば、UNIXドメインソケットを通じて scim-bridge
に接続するために、関数 `make-network-process' を用いる代わりにスクリプトを
使用することを強制します。このスクリプトは通常、バージョン22以降の Emacs には
必要ありません。")

(put 'scim-bridge-socket-script-path
     'variable-documentation
     "ソケットのためのスクリプトの置かれるディレクトリを指定します。このディ
レクトリが存在していなければ、自動的に作成されます。")

(put 'scim-bridge-socket-script-name
     'variable-documentation
     "スクリプトのファイル名を指定します。")

(put 'scim-bridge-socket-script
     'variable-documentation
     "スクリプトの内容です。このスクリプトは第1引数をソケット名として用いなけ
ればなりません。")

(put 'scim-meta-key-exists
     'variable-documentation
     "キーボードにmetaキーがある場合、この変数には t がセットされます。
自動検出がうまくいかない場合は、scim-bridge.el を読み込む前に手動で設定して
ください。")

(put 'scim-tmp-buffer-name
     'variable-documentation
     "エージェントと通信するためのワーキングバッファの名前です。")

(put 'scim-incompatible-mode-hooks
     'variable-documentation
     "これらのフックが実行される時には、scim-mode-map が無効化されます。")

(scim-replace-function-doc
 'scim-define-common-key
 "どのキーイベントが常時 SCIM へ送られるかを指定します。HANDLE が nil で
なければ、SCIM は KEY で与えられたキーイベントを処理します。KEY が配列として
与えられた場合は、キーシーケンスを表すのではなく、単独のキーストロークの複数
の定義を表します。
この設定を有効にするには、関数 `scim-update-key-bindings' を呼び出すか、
または scim-mode を再起動する必要があります。")

(scim-replace-function-doc
 'scim-define-preedit-key
 "どのキーイベントがプリエディット時に SCIM へ送られるかを指定します。
HANDLE が nil でなければ、SCIM は KEY で与えられたキーイベントを処理します。
KEY が配列として与えられた場合は、キーシーケンスを表すのではなく、単独のキー
ストロークの複数の定義を表します。
この設定を有効にするには、関数 `scim-update-key-bindings' を呼び出すか、
または scim-mode を再起動する必要があります。")

(scim-replace-function-doc
 'scim-get-frame-extents
 "フレームの外縁の幅のピクセル数を (left right top bottom) という形で返し
ます。ここで、`top' はフレームのタイトルバーの高さでもあります。")

(scim-replace-function-doc
 'scim-frame-header-height
 "メニューバーとツールバーの高さの合計をピクセル数で返します。この関数の返す
値はあまり正確ではありません。")

(scim-replace-function-doc
 'scim-real-frame-header-height
 "メニューバーとツールバーの高さの合計をピクセル数で返します。この関数の返す
値はとても正確ですが、関数 `scim-frame-header-height' と比べてずっと遅いです。")

(scim-replace-function-doc
 'scim-compute-pixel-position
 "ポイント位置のスクリーン上のピクセル座標を (X .Y) という形で返します。
この値はその文字の左下隅の座標を示しています。")

(scim-replace-function-doc
 'scim-get-gnome-font-size
 "GNOMEデスクトップ環境における、アプリケーションフォントの大きさのピクセル
数を返します。スクリーンの解像度が設定され、かつシェルコマンド `gconftool-2'
が使える必要があります。さもなければ、この関数はゼロを返します。")

(scim-replace-function-doc
 'scim-get-active-window-id
 "最前面にある、すなわち入力フォーカスのあるウインドウの、ウインドウシステム
におけるウインドウ番号を返します。")

(provide 'scim-bridge-ja)

;;;
;;; scim-bridge-ja.el ends here
